/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 *  @{
 */

/** @file
 */

#include <async.h>
#include <stdio.h>
#include <task.h>

#include <ipc/ipc.h>

#include "err.h"

extern int	module_message( ipc_callid_t callid, ipc_call_t * call, ipc_call_t * answer, int * answer_count );
extern void	module_print_name( void );
extern int	module_start( async_client_conn_t client_connection );

void	client_connection( ipc_callid_t iid, ipc_call_t * icall );
int	main( int argc, char * argv[] );

/** Default thread for new connections.
 */
void client_connection( ipc_callid_t iid, ipc_call_t * icall ){
	ipc_callid_t	callid;
	ipc_call_t	call;
	ipc_call_t	answer;
	int		count;
	int		res;

	/*
	 * Accept the connection
	 *  - Answer the first IPC_M_CONNECT_ME_TO call.
	 */
	ipc_answer_0( iid, EOK );

	while( true ){
		// refresh data
		count = 0;
		IPC_SET_RETVAL( answer, 0 );
		// just to be precize
		IPC_SET_RETVAL( answer, 0 );
		IPC_SET_ARG1( answer, 0 );
		IPC_SET_ARG2( answer, 0 );
		IPC_SET_ARG3( answer, 0 );
		IPC_SET_ARG4( answer, 0 );
		IPC_SET_ARG5( answer, 0 );

		callid = async_get_call( & call );
		res = module_message( callid, & call, & answer, & count );

		switch( count ){
			case 0:		ipc_answer_0( callid, res );
					continue;
			case 1:		ipc_answer_1( callid, res, IPC_GET_ARG1( answer ));
					continue;
			case 2:		ipc_answer_2( callid, res, IPC_GET_ARG1( answer ), IPC_GET_ARG2( answer ));
					continue;
			case 3:		ipc_answer_3( callid, res, IPC_GET_ARG1( answer ), IPC_GET_ARG2( answer ), IPC_GET_ARG3( answer ));
					continue;
			case 4:		ipc_answer_4( callid, res, IPC_GET_ARG1( answer ), IPC_GET_ARG2( answer ), IPC_GET_ARG3( answer ), IPC_GET_ARG4( answer ));
					continue;
			default:	ipc_answer_5( callid, res, IPC_GET_ARG1( answer ), IPC_GET_ARG2( answer ), IPC_GET_ARG3( answer ), IPC_GET_ARG4( answer ), IPC_GET_ARG5( answer ));
					continue;
		}
	}
}

/**	Starts the module.
 *	Parameters are ignored.
 */
int main( int argc, char * argv[] ){
	ERROR_DECLARE;

	printf("\nTask %d - ", task_get_id());
	module_print_name();
	if( ERROR_OCCURRED( module_start( client_connection ))){
		printf( " - ERROR %i", ERROR_CODE );
		return ERROR_CODE;
	}
	return EOK;
}

/** @}
 */
